/*============================================================================
  Summary:  Demonstrates Forwarding Records
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE master
GO

CREATE DATABASE ForwardingRecords
GO

USE ForwardingRecords
GO

SET STATISTICS IO ON
GO

-- Create a table to demonstrate forwarding records
CREATE TABLE HeapTable
(
	Col1 INT IDENTITY(1, 1),
	Col2 CHAR(2000),
	Col3 VARCHAR(1000)
)
GO

-- Insert 4 records - those will fit into one page
INSERT INTO HeapTable VALUES
(
	REPLICATE('1', 2000),
	''
),
(
	REPLICATE('2', 2000),
	''
),
(
	REPLICATE('3', 2000),
	''
),
(
	REPLICATE('4', 2000),
	''
)
GO

-- SQL Server will report 1 logical read
SELECT * FROM HeapTable
GO

-- Let's update the table and expand each row of the table
UPDATE HeapTable
SET Col3 = REPLICATE('5', 1000)
GO

-- SQL Server will report now 4 logical reads, because of the forwarding records
SELECT * FROM HeapTable
GO

-- Check the forwarding record count through sys.dm_db_index_physical_stats
SELECT
	index_type_desc,
	page_count, 
	avg_page_space_used_in_percent,
    avg_record_size_in_bytes,
    forwarded_record_count
FROM sys.dm_db_index_physical_stats
(
	DB_ID('ForwardingRecords'), 
    OBJECT_ID('HeapTable'),
    NULL, 
    NULL, 
    'DETAILED'
)
GO

-- Rebuild the heap table to get rid of the Forwarding Records
ALTER TABLE HeapTable REBUILD
GO

-- Check the Forwarding Records again
SELECT
	index_type_desc,
	page_count, 
	avg_page_space_used_in_percent,
    avg_record_size_in_bytes,
    forwarded_record_count
FROM sys.dm_db_index_physical_stats
(
	DB_ID('ForwardingRecords'), 
    OBJECT_ID('HeapTable'),
    NULL, 
    NULL, 
    'DETAILED'
)
GO

-- Clean up
USE master
GO

DROP DATABASE ForwardingRecords
GO